import nsrunit;       // imports unit file
unit conversion on;   // unit balance checking

math Comp1FlowExtra {      // 1 compartment model with flow
/* Diagram

 F(flow)      _____________________________
 Cin(t) ---> |                        C1(t)|---> Cout(t)=C1(t)
             | V1(volume)                  |
             | instantaneously well mixed  |
             |_____________________________|
*/
// independent variable will be t (time)
realDomain t sec; t.min=0; t.max=30; t.delta=0.1; 

// Parameters and variables
real C1(t) mM;                             // Concentration 
real C10 = 0 mM;                           // Initial Concentration 
extern real Cin(t) mM;                     // Inflow concentration
real Cout(t) mM;                           // Outflow Concentration
real V1 = 0.05 ml,                         // Volume  
     F  = 1 ml/min;                        // Flow 

// EXTRA Optional Clearance
choice Clearance ("no","yes")=1;           // Clearance choice
private int clear=Clearance;
real G  = 1 ml/min;

// Initial Condition
when(t=t.min) {C1=C10;}           

// The ordinary differential equation (ODE) 
C1:t = F/V1*(Cin-C1) + (if(clear=2) -G/V1*C1 else 0);

/* Outflow concentration equals compartment concentration
   because compartments are instantaneously well mixed.   */
Cout=C1;

// EXTRA
realState C1max(t) mM;
when(t=t.min) {C1max=0;}
event(C1max<C1) C1max=C1;

//Additional Calculations
real Q(t) mmol, Qint(t) mmol; when (t=t.min) {Qint=V1*C10;}
Q=V1*C1;
Qint:t=F*(Cin-Cout);

// STATISTICS
// Cin is the model input, Cout is the model output
private real S1_in(t) mM*sec, S2_in(t) mM*sec^2, S3_in(t) mM*sec^3;
when(t=t.min) {S1_in=0;S2_in=0;S3_in=0;}
real AREA_in(t) mM*sec, TBAR_in(t) sec, RD_in(t) ;
S1_in:t=Cin  ;
S2_in:t=Cin  *t;
S3_in:t=Cin  *t*t;
AREA_in=S1_in;
TBAR_in=S2_in/S1_in;
RD_in=sqrt((abs(S3_in/S2_in)/TBAR_in)-1);
//
private real S1_out(t) mM*sec, S2_out(t) mM*sec^2, S3_out(t) mM*sec^3;
when(t=t.min) {S1_out=0;S2_out=0;S3_out=0;}
real AREA_out(t) mM*sec, TBAR_out(t) sec, RD_out(t) ;
S1_out:t=Cout;
S2_out:t=Cout*t;
S3_out:t=Cout*t*t;
AREA_out=S1_out;
TBAR_out=S2_out/S1_out;
RD_out=sqrt((abs(S3_out/S2_out)/TBAR_out)-1);
//
private real SD_op(t);
SD_op= sqrt( (RD_out*TBAR_out)^2 - (RD_in*TBAR_in)^2 );
real TBAR_op(t), RD_op(t);
TBAR_op=TBAR_out-TBAR_in;
RD_op=SD_op/TBAR_op;

}
