import nsrunit;       // imports unit file
unit conversion on;   // unit balance checking

math Comp2Flow { // 2 compartment model with flow
/* Diagram

 F(flow)      _____________________________
 Cin(t) ---> |                        C1(t)|---> Cout(t)=C1(t)
             | V1(volume)                  |
             | instantaneously well mixed  |
             |             ^               |
             |____________PS_______________|
             |             v               |
             | V2(volume)             C2(t)|
             | instantaneously well mixed  |
             |_____________________________|
*/
// independent variable will be t (time)
realDomain t sec; t.min=0; t.max=30; t.delta=0.1; 

// Parameters and variables
real C1(t) mM,     C2(t) mM;               // Concentrations 
real C10 = 0 mM,   C20 = 0 mM;             // Initial Concentrations
extern real Cin(t) mM;                     // Inflow concentration
real Cout(t) mM;                           // Outflow Concentration
real V1 = 0.05 ml, V2=0.15 ml,             // Volumes 
     F  = 1 ml/min,                        // Flow 
     PS = 2 ml/min;                        // Exchange rate

// Initial Conditions
when(t=t.min) {C1=C10; C2=C20;}           

// The ordinary differential equations (ODE) 
C1:t = F/V1*(Cin-C1) +PS/V1*(C2-C1);
C2:t =               -PS/V2*(C2-C1);

/* Outflow concentration equals compartment concentration
   because compartments are instantaneously well mixed.   */
Cout=C1;


//Additional Calculations
real Q(t) mmol, Qint(t) mmol; when (t=t.min) {Qint=0;}
Q=V1*C1+V2*C2;
Qint:t=F*(Cin-Cout);

// STATISTICS
// Cin is the model input, Cout is the model output
private real S1_in(t) mM*sec, S2_in(t) mM*sec^2, S3_in(t) mM*sec^3;
when(t=t.min) {S1_in=0;S2_in=0;S3_in=0;}
real AREA_in(t) mM*sec, TBAR_in(t) sec, RD_in(t) ;
S1_in:t=Cin  ;
S2_in:t=Cin  *t;
S3_in:t=Cin  *t*t;
AREA_in=S1_in;
TBAR_in=S2_in/S1_in;
RD_in=sqrt((abs(S3_in/S2_in)/TBAR_in)-1);
//
private real S1_out(t) mM*sec, S2_out(t) mM*sec^2, S3_out(t) mM*sec^3;
when(t=t.min) {S1_out=0;S2_out=0;S3_out=0;}
real AREA_out(t) mM*sec, TBAR_out(t) sec, RD_out(t) ;
S1_out:t=Cout;
S2_out:t=Cout*t;
S3_out:t=Cout*t*t;
AREA_out=S1_out;
TBAR_out=S2_out/S1_out;
RD_out=sqrt((abs(S3_out/S2_out)/TBAR_out)-1);
//
private real SD_op(t);
SD_op= sqrt( (RD_out*TBAR_out)^2 - (RD_in*TBAR_in)^2 );
real TBAR_op(t), RD_op(t);
TBAR_op=TBAR_out-TBAR_in;
RD_op=SD_op/TBAR_op;

}
