import nsrunit;       // imports unit file
unit conversion on;   // unit balance checking

math Comp2FlowC2D {   // 2 compartment model with flow
                      //   and conversion of
                      //   C into D in 2nd compartment
/* Diagram

 F(flow)      _____________________________
 Cin(t) ---> |                 C1(t), D1(t)|---> Cout(t)=C1(t)
             | V1(volume)                  |---> Dout(t)=D1(t)
             | instantaneously well mixed  |
             |         ^        ^          |
             |________PSc______PSd_________|
             |         v        v          |
             |                             |
             | V2(volume)      C2(t), D2(t)|
             |                             |
             |      kf1                    |
             |      --->                   |
             |   C2 <--- D2                |
             |       kb1                   |
             | instantaneously well mixed  |
             |_____________________________|

*/
// independent variable will be t (time)
realDomain t sec; t.min=0; t.max=30; t.delta=0.1; 

// Parameters and variables
real C1(t) mM,     C2(t)  mM,               // Concentrations 
     D1(t) mM,     D2(t)  mM;

extern real Cin(t) mM;                     // Inflow concentration
real Cout(t) mM, Dout(t) mM;               // Outflow Concentration

real C10  = 0 mM,   C20 = 0 mM,            // Initial Concentrations
     D10  = 0 mM,   D20 = 0 mM;
real V1 = 0.05 ml, V2=0.15 ml,             // Volumes 
     F  = 1 ml/min,                        // Flow 
     PSc = 2 ml/min, PSd = 2 ml/min;       // Exchange rates

real kf= 300 sec^(-1),                     // Conversion reaction rates
     kb= 100 sec^(-1);

// Initial Conditions
when(t=t.min) {C1=C10; C2=C20;
               D1=D10; D2=D20;}           

// The ordinary differential equations (ODE) 
C1:t = F/V1*(Cin-C1) + PSc/V1*(C2-C1);
D1:t = F/V1*(   -D1) + PSd/V1*(D2-D1);

C2:t =            -PSc/V2*(C2-C1) -kf*C2+kb*D2;
/*
   IMPORTANT NOTE:
   Question: Why is PSc divided by V2 in the above equation, but
             kf and kb are not?
   Answer:   Multiply both sides by V2 to change the equation from 
             concentrations into amounts. Then -PSc*(C2-C1) is a 
             flux of material into and out of V2. The conversion terms 
             are changes in amounts in the second compartment.

*/

D2:t =            -PSd/V2*(D2-D1) +kf*C2  -kb*D2 ;
/* Outflow concentration equals compartment concentration
   because compartments are instantaneously well mixed.   */
Cout=C1;
Dout=D1;


//Additional Calculations
real Q(t) mmol, Qint(t) mmol; 
when (t=t.min) {Qint=V1*(C10+D10)+V2*(C20+D20);}
Q=V1*(C1+D1)+V2*(C2+D2);
Qint:t=F*(Cin-Cout-Dout);
}
